@@ -1,23 +1,7 @@
-/*
- * Copyright (C) 2020 ~ 2021 Uniontech Software Technology Co., Ltd.
- *
- * Author:     ZhangYong <zhangyong@uniontech.com>
- *
- * Maintainer: ZhangYong <ZhangYong@uniontech.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2020 - 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "movieservice.h"
 #include <QMetaType>
 #include <QDirIterator>
@@ -26,8 +10,13 @@
 #include <MediaInfoDLL/MediaInfoDLL.h>
 #include <QProcess>
 #include <QtDebug>
+#include <QJsonDocument>
 #include "service/gstreamervideothumbnailer.h"
 
+#ifdef EnableFFmpegLib
+#include "service/ffmpegvideothumbnailer.h"
+#endif
+
 #include <iostream>
 
 #define SEEK_TIME "00:00:01"
@@ -67,7 +56,7 @@ MovieService::MovieService(QObject *parent)
     : QObject(parent)
 {
     //检查ffmpeg是否存在
-    if(checkCommandExist("ffmpeg")) {
+    if (checkCommandExist("ffmpeg")) {
         resolutionPattern = "[0-9]+x[0-9]+";
         codeRatePattern = "[0-9]+\\skb/s";
         fpsPattern = "[0-9]+\\sfps";
@@ -75,9 +64,16 @@ MovieService::MovieService(QObject *parent)
     }
 
     //检查ffmpegthumbnailer是否存在
-    if(checkCommandExist("ffmpegthumbnailer")) {
+    if (checkCommandExist("ffmpegthumbnailer")) {
         m_ffmpegthumbnailerExist = true;
     }
+
+    //检查ffmpegthumbnailerlib是否存在
+#ifdef EnableFFmpegLib
+    if (initFFmpegVideoThumbnailer()) {
+        m_ffmpegThumLibExist = true;
+    }
+#endif
 }
 
 bool MovieService::checkCommandExist(const QString &command)
@@ -194,8 +190,8 @@ static QString searchLineFromKeyString(const std::string &key, const QString &ta
 MovieInfo MovieService::parseFromFile(const QFileInfo &fi)
 {
     auto info = getMovieInfo_mediainfo(fi);
-    if(!info.valid) {
-        if(m_ffmpegExist) {
+    if (!info.valid) {
+        if (m_ffmpegExist) {
             info = getMovieInfo_ffmpeg(fi);
         } else {
             info.filePath = fi.absoluteFilePath();
@@ -208,20 +204,41 @@ MovieInfo MovieService::parseFromFile(const QFileInfo &fi)
     return info;
 }
 
-void getMovieCover(const QUrl &url, const QString &savePath, QImage *imageRet)
-{
-    *imageRet = MovieService::instance()->getMovieCover(url, savePath);
-}
-
 QImage MovieService::getMovieCover(const QUrl &url, const QString &savePath)
 {
-    auto image = getMovieCover_gstreamer(url);
-    if(image.isNull() && m_ffmpegthumbnailerExist) {
+    QImage image;
+
+#ifdef EnableFFmpegLib
+    if (m_ffmpegThumLibExist) {
+        image = getMovieCover_ffmpegthumbnailerlib(url);
+        if (!image.isNull()) {
+            return image;
+        }
+    }
+#endif
+
+    image = getMovieCover_gstreamer(url);
+    if (image.isNull() && m_ffmpegthumbnailerExist) {
         image = getMovieCover_ffmpegthumbnailer(url, savePath);
     }
     return image;
 }
 
+#ifdef EnableFFmpegLib
+
+QImage MovieService::getMovieCover_ffmpegthumbnailerlib(const QUrl &url)
+{
+    QImage image;
+
+    if (!m_ffmpegThumLibExist) {
+        return image;
+    }
+
+    return runFFmpegVideoThumbnailer(url);
+}
+
+#endif
+
 QImage MovieService::getMovieCover_ffmpegthumbnailer(const QUrl &url, const QString &bufferPath)
 {
     QImage image;
@@ -256,8 +273,97 @@ QImage MovieService::getMovieCover_ffmpegthumbnailer(const QUrl &url, const QStr
     return image;
 }
 
+std::vector<std::pair<QString, QString>> searchGroupFromKey(const QString &key, const QStringList &datas)
+{
+    std::vector<std::pair<QString, QString>> result;
+
+    int i = 0;
+
+    //搜索起始位置
+    for (; i != datas.size(); ++i) {
+        if (datas[i] == key) {
+            ++i;
+            break;
+        }
+    }
+
+    //读取数据
+    for (; i != datas.size(); ++i) {
+        if (datas[i].isEmpty()) {
+            break;
+        }
+
+        auto tokens = datas[i].split(",");
+        if (tokens.size() < 2) { //数据异常
+            continue;
+        }
+        result.push_back(std::make_pair(tokens[0], tokens[1]));
+    }
+
+    return result;
+}
+
+bool MovieService::isCrashFormat(const QUrl &url)
+{
+    QFileInfo fi(url.toLocalFile());
+    MediaInfoDLL::MediaInfo info;
+    info.Open(fi.absoluteFilePath().toStdString());
+    info.Option("Complete", "1"); //解析全部数据
+    info.Option("Inform", "CSV"); //CSV格式输出
+    QString strData(info.Inform().c_str());
+
+    //至此视频信息已保存在infoList中，每条数据以key开头，以冒号分隔
+    QStringList infoList = strData.split("\n");
+
+    //1.搜索特征数据
+    auto videoGroup = searchGroupFromKey("Video", infoList);
+    if (videoGroup.empty()) {
+        return true;
+    }
+
+    //1.1 code format = mpv
+    QString codeID;
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Internet media type") {
+            auto str = eachPair.second;
+            auto list = str.split("/");
+            if (list.size() == 2) {
+                codeID = list[1].toLower();
+            } else {
+                codeID = str.toLower();
+            }
+            break;
+        }
+    }
+
+    //1.2 package format = mkv
+    auto generalGroup = searchGroupFromKey("General", infoList);
+    QStringList packageIds;
+    for (auto &eachPair : generalGroup) {
+        if (eachPair.first == "Format/Extensions usually used") {
+            auto str = eachPair.second;
+            packageIds = str.split(" ");
+            break;
+        }
+    }
+
+    //2.特征比对
+    if (codeID == "mpv" && packageIds.contains("mkv")) {
+        return true;
+    }
+
+    return false;
+}
+
 QImage MovieService::getMovieCover_gstreamer(const QUrl &url)
 {
+    //ARM环境需要额外判断以规避崩溃问题
+#ifdef EnableFFmpegLib
+    if (isCrashFormat(url)) {
+        return QImage();
+    }
+#endif
+
     return runGstreamerVideoThumbnailer(url);
 }
 
@@ -265,7 +371,7 @@ MovieInfo MovieService::getMovieInfo_ffmpeg(const QFileInfo &fi)
 {
     struct MovieInfo mi;
 
-    if(!m_ffmpegExist) {
+    if (!m_ffmpegExist) {
         mi.valid = false;
         return mi;
     }
@@ -386,34 +492,50 @@ MovieInfo MovieService::getMovieInfo_ffmpeg(const QFileInfo &fi)
     return mi;
 }
 
-std::vector<std::pair<QString, QString>> searchGroupFromKey(const QString &key, const QStringList &datas)
+template <typename T>
+void checkIfNotDoInsert(QJsonObject *jsonObj, const QString &jsonKey, T data, T value)
 {
-    std::vector<std::pair<QString, QString>> result;
+    if (data != value) {
+        jsonObj->insert(jsonKey, data);
+    }
+}
 
-    int i = 0;
+QJsonObject MovieService::getMovieInfoByJson(const QUrl &url)
+{
+    QJsonObject jsonRet;
 
-    //搜索起始位置
-    for(;i != datas.size();++i) {
-        if(datas[i] == key) {
-            ++i;
-            break;
-        }
+    auto info = getMovieInfo(url);
+    if (info.valid) {
+        QJsonObject jsonObjBase;
+        checkIfNotDoInsert(&jsonObjBase, "FilePath", info.filePath, QString("-"));
+        checkIfNotDoInsert(&jsonObjBase, "FileType", info.fileType, QString("-"));
+        checkIfNotDoInsert(&jsonObjBase, "Resolution", info.resolution, QString("-"));
+        checkIfNotDoInsert(&jsonObjBase, "Creation", info.creation.toString(Qt::ISODate), QString());
+        checkIfNotDoInsert(&jsonObjBase, "FileSize", info.fileSize, 0ll);
+        checkIfNotDoInsert(&jsonObjBase, "Duration", info.duration, QString("-"));
+
+        QJsonObject jsonObjVideo;
+        checkIfNotDoInsert(&jsonObjVideo, "CodecID", info.vCodecID, QString("-"));
+        checkIfNotDoInsert(&jsonObjVideo, "CodeRate", info.vCodeRate, 0ll);
+        checkIfNotDoInsert(&jsonObjVideo, "Fps", info.fps, 0);
+        checkIfNotDoInsert(&jsonObjVideo, "Proportion", info.proportion, -1.0);
+
+        QJsonObject jsonObjAudio;
+        checkIfNotDoInsert(&jsonObjAudio, "CodecID", info.aCodeID, QString("-"));
+        checkIfNotDoInsert(&jsonObjAudio, "CodeRate", info.aCodeRate, 0ll);
+        checkIfNotDoInsert(&jsonObjAudio, "ChannelCount", info.channels, 0);
+        checkIfNotDoInsert(&jsonObjAudio, "Sampling", info.sampling, 0);
+
+        jsonRet.insert("Base", jsonObjBase);
+        jsonRet.insert("Video", jsonObjVideo);
+        jsonRet.insert("Audio", jsonObjAudio);
+
+        checkIfNotDoInsert(&jsonRet, "Base", jsonObjBase, QJsonObject());
+        checkIfNotDoInsert(&jsonRet, "Video", jsonObjVideo, QJsonObject());
+        checkIfNotDoInsert(&jsonRet, "Audio", jsonObjAudio, QJsonObject());
     }
 
-    //读取数据
-    for(;i != datas.size();++i) {
-        if(datas[i].isEmpty()) {
-            break;
-        }
-
-        auto tokens = datas[i].split(",");
-        if(tokens.size() < 2) { //数据异常
-            continue;
-        }
-        result.push_back(std::make_pair(tokens[0], tokens[1]));
-    }
-
-    return result;
+    return jsonRet;
 }
 
 MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
@@ -436,22 +558,25 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
     mi.fileType = fi.suffix().toLower();
     mi.fileSize = fi.size();
 
-    //2.2.视频流数据
+    //2.3.视频流数据
     auto videoGroup = searchGroupFromKey("Video", infoList);
-    
-    if(!videoGroup.empty()) {
+
+    if (!videoGroup.empty()) {
         mi.valid = true;
     } else {
         mi.valid = false;
         return mi;
     }
 
+    //全局数据备用
+    auto generalGroup = searchGroupFromKey("General", infoList);
+
     //编码格式
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first == "Internet media type") {
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Internet media type") {
             auto str = eachPair.second;
             auto list = str.split("/");
-            if(list.size() == 2) {
+            if (list.size() == 2) {
                 mi.vCodecID = list[1].toLower();
             } else {
                 mi.vCodecID = str.toLower();
@@ -460,37 +585,46 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
         }
     }
 
+    if (mi.vCodecID == "-") {
+        for (auto &eachPair : videoGroup) {
+            if (eachPair.first == "Format") {
+                mi.vCodecID = eachPair.second;
+                break;
+            }
+        }
+    }
+
     //分辨率，长宽比
     int width = 0;
     int height = 0;
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first == "Width") {
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Width") {
             width = eachPair.second.toInt();
-            if(width > 0) {
+            if (width > 0) {
                 break;
             }
         }
     }
 
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first == "Height") {
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Height") {
             height = eachPair.second.toInt();
-            if(height > 0) {
+            if (height > 0) {
                 break;
             }
         }
     }
 
-    if(width > 0 && height > 0) {
+    if (width > 0 && height > 0) {
         mi.resolution = QString::number(width) + "x" + QString::number(height);
         mi.proportion = static_cast<double>(width) / height;
     }
 
     //码率
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first.indexOf("bit rate", Qt::CaseInsensitive) != -1) {
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first.indexOf("Bit rate", Qt::CaseInsensitive) != -1) {
             auto codeRate = eachPair.second.toInt();
-            if(codeRate > 0) {
+            if (codeRate > 0) {
                 mi.vCodeRate = codeRate / 1000;
                 break;
             }
@@ -498,43 +632,76 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
     }
 
     //帧率
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first == "Frame rate") {
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Frame rate") {
             double fps = eachPair.second.toDouble();
-            if(fps > 0) {
+            if (fps > 0) {
                 mi.fps = static_cast<int>(fps);
                 break;
             }
         }
     }
 
-    //2.3.时长数据
-    for(auto &eachPair : videoGroup) {
-        if(eachPair.first == "Duration") {
+    if (mi.fps == 0) {
+        for (auto &eachPair : generalGroup) {
+            if (eachPair.first == "Frame rate") {
+                double fps = eachPair.second.toDouble();
+                if (fps > 0) {
+                    mi.fps = static_cast<int>(fps);
+                    break;
+                }
+            }
+        }
+    }
+
+    //2.4.时长数据
+    for (auto &eachPair : videoGroup) {
+        if (eachPair.first == "Duration") {
             auto list = eachPair.second.split(".");
-            if(list.size() == 2 && std::count(list[0].begin(), list[0].end(), ':') >= 2) {
+            if (list.size() == 2 && std::count(list[0].begin(), list[0].end(), ':') >= 2) {
                 mi.duration = list[0];
                 break;
             }
         }
     }
 
-    //2.4.音频数据
+    if (mi.duration == "-") {
+        for (auto &eachPair : generalGroup) {
+            if (eachPair.first == "Duration") {
+                auto list = eachPair.second.split(".");
+                if (list.size() == 2 && std::count(list[0].begin(), list[0].end(), ':') >= 2) {
+                    mi.duration = list[0];
+                    break;
+                }
+            }
+        }
+    }
+
+    //2.5.创建时间
+    auto geGroup = searchGroupFromKey("General", infoList);
+    for (auto &eachPair : geGroup) {
+        if (eachPair.first == "Recorded date") {
+            mi.creation = QDateTime::fromString(eachPair.second, Qt::ISODateWithMs);
+            break;
+        }
+    }
+
+    //2.6.音频数据
     auto audioGroup = searchGroupFromKey("Audio", infoList);
 
     //编码格式
-    for(auto &eachPair : audioGroup) {
-        if(eachPair.first == "Commercial name") {
+    for (auto &eachPair : audioGroup) {
+        if (eachPair.first == "Commercial name") {
             mi.aCodeID = eachPair.second.toLower();
             break;
         }
     }
 
     //采样率
-    for(auto &eachPair : audioGroup) {
-        if(eachPair.first == "Sampling rate") {
+    for (auto &eachPair : audioGroup) {
+        if (eachPair.first == "Sampling rate") {
             int sampling = eachPair.second.toInt();
-            if(sampling > 0) {
+            if (sampling > 0) {
                 mi.sampling = sampling;
                 break;
             }
@@ -542,10 +709,10 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
     }
 
     //通道数
-    for(auto &eachPair : audioGroup) {
-        if(eachPair.first == "Channel(s)") {
+    for (auto &eachPair : audioGroup) {
+        if (eachPair.first == "Channel(s)") {
             int channels = eachPair.second.toInt();
-            if(channels > 0) {
+            if (channels > 0) {
                 mi.channels = channels;
                 break;
             }
@@ -553,10 +720,10 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
     }
 
     //码率
-    for(auto &eachPair : audioGroup) {
-        if(eachPair.first == "Bit rate") {
+    for (auto &eachPair : audioGroup) {
+        if (eachPair.first == "Bit rate") {
             int codeRate = eachPair.second.toInt();
-            if(codeRate > 0) {
+            if (codeRate > 0) {
                 mi.aCodeRate = codeRate / 1000;
                 break;
             }
@@ -564,10 +731,25 @@ MovieInfo MovieService::getMovieInfo_mediainfo(const QFileInfo &fi)
     }
 
     //音频位数（无法读出，待移除）
-    if(mi.aCodeID != "-") {
+    if (mi.aCodeID != "-") {
         mi.aDigit = "8 bits";
     }
 
     //返回最终解析结果
     return mi;
 }
+
+//C-Style API
+extern "C" {
+
+    void getMovieCover(const QUrl &url, const QString &savePath, QImage *imageRet)
+    {
+        *imageRet = MovieService::instance()->getMovieCover(url, savePath);
+    }
+
+    void getMovieInfoByJson(const QUrl &url, QJsonObject *jsonRet)
+    {
+        *jsonRet = MovieService::instance()->getMovieInfoByJson(url);
+    }
+
+}
