@@ -1,23 +1,7 @@
-/*
- * Copyright (C) 2020 ~ 2021 Uniontech Software Technology Co., Ltd.
- *
- * Author:     ZhangYong <zhangyong@uniontech.com>
- *
- * Maintainer: ZhangYong <ZhangYong@uniontech.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2020 - 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "unionimage.h"
 #include <FreeImage.h>
 //#include "giflib/cmanagerattributeservice.h"
@@ -47,6 +31,9 @@
 const QString DATETIME_FORMAT_NORMAL = "yyyy.MM.dd";
 const QString DATETIME_FORMAT_EXIF = "yyyy:MM:dd HH:mm";
 
+// 提供用于非线程安全函数 FreeImage_TagToString() 访问限制的锁
+Q_GLOBAL_STATIC(QMutex, g_freeImageTagToStringMutex);
+
 namespace LibUnionImage_NameSpace {
 
 //    enum SupportFormat {
@@ -343,8 +330,15 @@ UNIONIMAGESHARED_EXPORT QMap<QString, QString> getMetaData(FREE_IMAGE_MDMODEL mo
         mdhandle = FreeImage_FindFirstMetadata(model, dib, &tag);
         if (mdhandle) {
             do {
-                mdMap.insert(FreeImage_GetTagKey(tag),
-                             FreeImage_TagToString(model, tag));
+                QString value;
+                // FreeImage_TagToString非线程安全，使用前加锁保护
+                if (g_freeImageTagToStringMutex.exists()) {
+                    g_freeImageTagToStringMutex->lock();
+                    value = QString(FreeImage_TagToString(model, tag));
+                    g_freeImageTagToStringMutex->unlock();
+                }
+
+                mdMap.insert(FreeImage_GetTagKey(tag), value);
             } while (FreeImage_FindNextMetadata(mdhandle, &tag));
             FreeImage_FindCloseMetadata(mdhandle);
         }
